# LSB radix sort in optimized assembly (gcc 45_Sort.c 45_Sort2.s)

        .intel_syntax noprefix
        .globl     Sort

Sort:  
# Allocating an extra array (because sorting is not in place).
        xchg       rsi, rdi
        shl        rdi, 2
        push       rdi
        push       rsi
        call       malloc                # It can change the rdi, rsi registers
        mov        rdx, rax
        pop        rsi
        pop        rdi
        # The address of the original array is in rsi, the address of the temporary array is in rdx.

# Looking for the grateast value in the array.
        add        rdi, rsi              # The address of the memory cell beyond the original array is in rdi. 
        mov        eax, DWORD PTR [rsi]
        lea        r8, [rsi+4]           # The address of the current element of the original array is in r8.
        jmp        .cond1
.body1: cmp        eax, DWORD PTR [r8]
        jae        .next1
        mov        eax, DWORD PTR [r8]
.next1: add        r8, 4
.cond1: cmp        r8, rdi
        jb         .body1
        # The greatest array element is in the eax.

# Finding the most significant '1' bit (order of magnitude of the maximum)
        mov        ecx, 0xffffffff
.body2: inc        ecx
        shr        eax, 1
        jnz        .body2
        mov        eax, 1
        shl        eax, cl
        # The most significant bit is specified by the mask stored in eax.

# Sort by binary LSB radix sort algoritm not in place.
        mov       ebx, 1                 # The corrent mask (LSB)

   # Copy the values with '0' bit to the beginning of the temporary array.
.nextdigit:
        mov       r9, rdx                # The address of the current element of the temporary array is in r9.
        mov       r8, rsi                # The address of the current element of the original array is in r8.
        jmp       .cond3
.body3: mov       ecx, DWORD PTR [r8]
        test      ecx, ebx
        jnz       .next3
        mov       DWORD PTR [r9], ecx
        add       r9, 4
.next3: add       r8, 4
.cond3: cmp       r8, rdi
        jb        .body3

   # Copy the values with '1' bit to the end of the temporary array.
        mov       r8, rsi                # The address of the current element of the original array is in r8.
        jmp       .cond4
.body4: mov       ecx, DWORD PTR [r8]
        test      ecx, ebx
        jz        .next4
        mov       DWORD PTR [r9], ecx
        add       r9, 4
.next4: add       r8, 4
.cond4: cmp       r8, rdi
        jb        .body4

   # Copy the temporary array back to the original array.
        mov       r9, rdx                # The address of the current element of the temporary array is in r9.
        mov       r8, rsi                # The address of the current element of the original array is in r8.
        jmp       .cond5
.body5: mov       ecx, DWORD PTR [r9] 
        mov       DWORD PTR [r8], ecx
        add       r8, 4
        add       r9, 4
.cond5: cmp       r8, rdi
        jb        .body5

    # Change the mask to the next digit.
        shl       ebx, 1
        cmp       ebx, eax
        jbe       .nextdigit

# Freeing the temporary array. 
        mov rdi, rdx
        call free
        ret
